  #include <EEPROM.h>
#include <LiquidCrystal.h>
#include <TimerOne.h>
#include <avr/pgmspace.h>

//LCD
static int Blink_counter = 0;
static boolean SP1Dots = true;
static boolean SP2Dots = true;
static LiquidCrystal LCD_1(3, 4, 5, 6, 7, 8, 9);
static LiquidCrystal LCD_2(2, A0, A1, A2, A3, A4, A5);

//Knöpfe
static boolean iSP1B = false;
static boolean iSP2B = false;
static boolean Running = false;
static boolean isPaused = false;
static boolean pausePlayer = false;

//Alles für die Modi
static int mode = 0;
static const int P1 PROGMEM = A6;
static const int P2 PROGMEM = A7;
static const int modecount PROGMEM = 4;
static int usermodecount = 0;

const char m1[] PROGMEM = "30M,5SZ";
const char m2[] PROGMEM = "10M,5SZ";
const char m3[] PROGMEM = "30M,0SZ";
const char m4[] PROGMEM = "10M,0SZ";

static PGM_P const modi[modecount] PROGMEM = {
  m1,
  m2,
  m3,
  m4,
};

const char s1[] PROGMEM = "Stunden";
const char s2[] PROGMEM = "Minuten";
const char s3[] PROGMEM = "Sekunden";
const char s4[] PROGMEM = "Zuschlag";

static PGM_P const msg[4] PROGMEM = {
  s1,
  s2,
  s3,
  s4, 
};

static const int ModeTime[modecount] [4] PROGMEM = {
  //Zeiten in Format:
  //Sekunden, Minuten, Stunden, Zuschlag
  //30M,5SZ
  { 00,       30,      00,      05},
  //10M,5SZ
  { 00,       10,      00,      05},
  //30M,0SZ
  { 00,       30,      00,      00},
  //10M,0SZ
  { 00,       10,      00,      00},
};

static int tempTimeData[4];

//Dreieckssymbol für die Auswahl des Moduses
static const byte cChar1[8] PROGMEM = {
  B10000,
  B11000,
  B11100,
  B11110,
  B11110,
  B11100,
  B11000,
  B10000,
};

byte Char[8];

//Zeit initialisierer
static int Zeit_Sekunden = 0;
static int Zeit_Minuten = 0;
static int Zeit_Stunden = 0;
static int ZuschlagProZug_Sekunden = 0;

//Zeit der Spieler
//Spieler 1
static int SP1Zeit_Sekunden = 0;
static int SP1Zeit_Minuten = 0;
static int SP1Zeit_Stunden = 0;

//Spieler 2
static int SP2Zeit_Sekunden = 0;
static int SP2Zeit_Minuten = 0;
static int SP2Zeit_Stunden = 0;

//Läuft die Uhr der Spieler?
static boolean SP1Lauf = false;
static boolean SP2Lauf = false;

//Initialisierung
void setup() {
  pinMode(P1, INPUT);
  pinMode(P2, INPUT);
  pinMode(13, OUTPUT);
  LCD_1.begin(8, 1);
  LCD_1.clear();
  LCD_2.begin(8, 1);
  LCD_2.clear();
  convertChar(cChar1);
  LCD_1.createChar(0, Char);
  usermodecount = EEPROMReadInt(0);
  if(usermodecount < 0) {
    usermodecount = 0;  
  }
}

//Modus auswählen
void loop() {
smode:
  int allmodecount = modecount + usermodecount;
  LCD_1.setCursor(0, 0);
  LCD_1.write(byte(0));
  if (mode < allmodecount) {
    if(mode < modecount) {
      char buffers[7];
      strcpy_P(buffers, (PGM_P)pgm_read_word(&(modi[mode])));
      LCD_1.print(buffers);
    }
    else {
      LCD_1.clear();
      LCD_1.write(byte(0));
      LCD_1.print(mode - modecount);
    }
  }
  if (mode == allmodecount) {
    LCD_1.print(F("+       "));
  }
  if (mode == allmodecount + 1) {
    LCD_1.print(F("-       "));
  }
  if (waitkeyPress(P1, P2, 0) == 1) {
    mode++;
    if (mode > allmodecount + 1) {
      mode = 0;
    }
    goto smode;
  }
  else {
    LCD_1.setCursor(0, 0);
    LCD_1.print(F("OK!     "));
    delay(500);
    LCD_1.setCursor(0, 0);
    //Modus ausgewählt
    if(mode < allmodecount) {
      if(mode < modecount) {
        prepareMode(mode);
        play();
      }
      //Höherer Modus als standard-modus Zahl ausgewählt, d.h. custom-modus ausgewählt!
      else {
        for(int i = 1; i < 5; i++) {
          tempTimeData[i - 1] = EEPROMReadInt((mode - modecount) * 4 + i);
        }
        Zeit_Stunden = tempTimeData[0];
        Zeit_Minuten = tempTimeData[1];
        Zeit_Sekunden = tempTimeData[2];
        ZuschlagProZug_Sekunden = tempTimeData[3];
        play();
      }
    }
    //"+"-Symbol (neuer Modus) ausgewählt
    if(mode == allmodecount) {
      enterModeParams();
    }
    //"-"-Symbol (Modus löschen) ausgewählt
    if(mode == allmodecount + 1) {
      deleteModes();
    }
  }
}

//Modus-einstellungen abfragen
void enterModeParams() {
  LCD_1.clear();
  LCD_2.clear();
  char buffers[8];
  for(int i = 0; i < 4; i++) {
    strcpy_P(buffers, (PGM_P)pgm_read_word(&(msg[i])));
    tempTimeData[i] = LCD_Selector(buffers);
  }
  usermodecount++;
  EEPROMWriteInt(0, usermodecount);
  for(int i = 1; i < 5; i++) {
    EEPROMWriteInt((usermodecount - 1) * 4 + i, tempTimeData[i - 1]);
  }
  Zeit_Stunden = tempTimeData[0];
  Zeit_Minuten = tempTimeData[1];
  Zeit_Sekunden = tempTimeData[2];
  ZuschlagProZug_Sekunden = tempTimeData[3];
  play();
}

int LCD_Selector(String msgs) {
  int number = 0;
  LCD_2.clear();
  LCD_2.print(msgs);
changed:
  LCD_1.setCursor(0, 0);
  if (number > 60) {
    number = 0;
  }
  LCD_1.print(getTimeString(number));
  if (waitkeyPress(P1, P2, 0) == 1) {
    number++;
    goto changed;
  }
  else {
    return number;
  }
}

String getTimeString(int time) {
  String returnTime;
  if (time < 10) {
    returnTime = "0" + String(time);
  }
  else {
    returnTime = String(time);
  }
  return returnTime;
}

//Modus Konfigurieren
void prepareMode(int Mode) {
  Zeit_Sekunden = pgm_read_byte(&(ModeTime[Mode][0]));
  Zeit_Minuten = pgm_read_byte(&(ModeTime[Mode][1]));
  Zeit_Stunden = pgm_read_byte(&(ModeTime[Mode][2]));
  ZuschlagProZug_Sekunden = pgm_read_byte(&(ModeTime[Mode][3]));
}

//Neustart-Funktion
void(* reboot) (void) = 0;

//Modi löschen dialog
void deleteModes() {
  LCD_1.clear();
  LCD_1.print(F("Sicher?"));
  boolean sure = false;
  while(true) {
    LCD_2.clear();
    if(sure) {
      LCD_2.print(F("Ja"));
    }
    else {
      LCD_2.print(F("Nein"));
    }
    if(waitkeyPress(P1, P2, 0) == 1) {
      sure = !sure;
    }
    else {
      break;
    }
  }
  LCD_1.clear();
  LCD_2.clear();
  if(sure) {
    LCD_1.print(F("Arbeitet"));
    EEPROMClearAll();
    LCD_1.clear();
    LCD_1.print(F("Fertig!"));
    delay(2000);
    reboot();
  }
}

//Manager fürs Spiel
void play() {
  //Zeit der Spieler initialisieren
  SP1Zeit_Sekunden = Zeit_Sekunden;
  SP2Zeit_Sekunden = Zeit_Sekunden;
  SP1Zeit_Minuten = Zeit_Minuten;
  SP2Zeit_Minuten = Zeit_Minuten;
  SP1Zeit_Stunden = Zeit_Stunden;
  SP2Zeit_Stunden = Zeit_Stunden;

  //Sekundentakter-Timer initialisieren
  Timer1.initialize(100000);
  Timer1.attachInterrupt(counter);

  LCD1_PrintTime(true);
  while (true) {
    //Knopf-Stati lesen
    boolean p1 = digitRead(P1);
    boolean p2 = digitRead(P2);

    if(Running) {
      digitalWrite(13, HIGH);
    }
    else {
      digitalWrite(13, LOW);
    }
    
    //Vielleicht drückt der Nutzer beide Knöpfe. Lasse ihm 100ms Zeit, um den 2. Knopf zu drücken.
    if(p1|p2) {
      for(int i = 0; i < 100; i++) {
        if(p1 & p2) {
          break;  
        }
        delay(1); 
      } 
    }

    if (p1 & p2 & !isPaused & !iSP1B & !iSP2B & Running) {
      isPaused = true;
      pausePlayer = SP2Lauf;
      SP1Lauf = false;
      SP2Lauf = false;
      iSP1B = true;
      iSP2B = true;
    }

    if ((p1 | p2) & isPaused & !iSP1B & !iSP2B & Running) {
      isPaused = false;
      SP2Lauf = pausePlayer;
      SP1Lauf = !pausePlayer;
      iSP1B = true;
      iSP2B = true;
    }


    //Knopf 1 gedrückt
    if (p1 & !p2& !iSP1B & !isPaused & (SP1Lauf | !Running | ((SP1Zeit_Stunden + SP1Zeit_Minuten + SP1Zeit_Sekunden < 1) & (SP2Zeit_Stunden + SP2Zeit_Minuten + SP2Zeit_Sekunden > 0)))) {
      SP1Lauf = false;
      SP2Lauf = true;
      if (Running & (SP1Zeit_Stunden + SP1Zeit_Minuten + SP1Zeit_Sekunden > 0)) {
        AddSeconds(false, ZuschlagProZug_Sekunden);
      }
      iSP1B = true;
      Running = true;
    }
    
    //Knopf 2 gedrückt
    if (p2 & !p1 & !iSP2B & !isPaused & (SP2Lauf | !Running | ((SP2Zeit_Stunden + SP2Zeit_Minuten + SP2Zeit_Sekunden < 1) & (SP1Zeit_Stunden + SP1Zeit_Minuten + SP1Zeit_Sekunden > 0)))) {
      SP2Lauf = false;
      SP1Lauf = true;
      if (Running & (SP2Zeit_Stunden + SP2Zeit_Minuten + SP2Zeit_Sekunden > 0)) {
        AddSeconds(true, ZuschlagProZug_Sekunden);
      }
      iSP2B = true;
      Running = true;
    }

    //Knopf 1 losgelassen
    if (!p1) {
      iSP1B = false;
    }

    //Knopf 2 losgelassen
    if (!p2) {
      iSP2B = false;
    }
    LCD1_PrintTime(SP1Dots);
    LCD2_PrintTime(SP2Dots);
  }
}

//Zeit von LCD1 ausgeben
void LCD1_PrintTime(boolean eSP1Dots) {
  String spacer = eSP1Dots ? ":" : " ";
  String T_Sek = String(SP1Zeit_Sekunden);
  String T_Min = String(SP1Zeit_Minuten);
  String T_Std = String(SP1Zeit_Stunden);
  if (T_Sek.toInt() < 10) {
    T_Sek = "0" + T_Sek;
  }
  if (T_Min.toInt() < 10) {
    T_Min = "0" + T_Min;
  }
  if (T_Std.toInt() < 10) {
    T_Std = "0" + T_Std;
  }
  LCD_1.setCursor(0, 0);
  if ((SP1Zeit_Minuten > 20) | (SP1Zeit_Stunden > 0)) {
    LCD_1.print(T_Std + spacer + T_Min + "   ");
  }
  else {
    LCD_1.print(T_Min + spacer + T_Sek + "   ");
  }
}

//Zeit von LCD2 ausgeben
void LCD2_PrintTime(boolean eSP2Dots) {
  String spacer = eSP2Dots ? ":" : " ";
  String T_Sek = String(SP2Zeit_Sekunden);
  String T_Min = String(SP2Zeit_Minuten);
  String T_Std = String(SP2Zeit_Stunden);
  if (T_Sek.toInt() < 10) {
    T_Sek = "0" + T_Sek;
  }
  if (T_Min.toInt() < 10) {
    T_Min = "0" + T_Min;
  }
  if (T_Std.toInt() < 10) {
    T_Std = "0" + T_Std;
  }
  LCD_2.setCursor(0, 0);
  if ((SP2Zeit_Minuten > 20) | (SP2Zeit_Stunden > 0)) {
    LCD_2.print(T_Std + spacer + T_Min + "   ");
  }
  else {
    LCD_2.print(T_Min + spacer + T_Sek + "   ");
  }
}

//Timer
//[Pro Millisekunde]
void counter() {
  Blink_counter++;
  if (Blink_counter == 10) {
    if (SP1Zeit_Sekunden + SP1Zeit_Minuten + SP1Zeit_Stunden < 1) {
      SP1Lauf = false;
    }
    if (SP2Zeit_Sekunden + SP2Zeit_Minuten + SP2Zeit_Stunden < 1) {
      SP2Lauf = false;
    }
    SP1Dots = true;
    SP2Dots = true;
    Blink_counter = 0;
    if (SP1Lauf) {
      SP1Zeit_Sekunden--;
      if (SP1Zeit_Sekunden < 0) {
        SP1Zeit_Minuten--;
        SP1Zeit_Sekunden = 59;
        if (SP1Zeit_Minuten < 0) {
          SP1Zeit_Minuten = 59;
          SP1Zeit_Stunden--;
        }
      }
    }
    if (SP2Lauf) {
      SP2Zeit_Sekunden--;
      if (SP2Zeit_Sekunden < 0) {
        SP2Zeit_Minuten--;
        SP2Zeit_Sekunden = 59;
        if (SP2Zeit_Minuten < 0) {
          SP2Zeit_Minuten = 59;
          SP2Zeit_Stunden--;
        }
      }
    }
  }

  if (Blink_counter == 5) {
    if (SP1Lauf & (SP1Zeit_Sekunden + SP1Zeit_Minuten + SP1Zeit_Stunden > 0)) {
      SP1Dots = false;
    }
    if (SP2Lauf & (SP2Zeit_Sekunden + SP2Zeit_Minuten + SP2Zeit_Stunden > 0)) {
      SP2Dots = false;
    }
  }
}

//Sekunden zu Spieler hinzufügen, false für Spieler 1 und true für Spieler 2
void AddSeconds(boolean player, int count) {
  for (int i = 0; i < count; i++)
    if (player) {
      //Spieler 2
      SP2Zeit_Sekunden++;
      if (SP2Zeit_Sekunden > 59) {
        SP2Zeit_Minuten++;
        SP2Zeit_Sekunden = 0;
        if (SP2Zeit_Minuten > 59) {
          SP2Zeit_Minuten = 0;
          SP2Zeit_Stunden++;
        }
      }
    }
    else {
      //Spieler 1
      SP1Zeit_Sekunden++;
      if (SP1Zeit_Sekunden > 59) {
        SP1Zeit_Minuten++;
        SP1Zeit_Sekunden = 0;
        if (SP1Zeit_Minuten > 59) {
          SP1Zeit_Minuten = 0;
          SP1Zeit_Stunden++;
        }
      }
    }
}

//Knopf mit Timeout abfrage-Funktion
int waitkeyPress(int keytowait, int abortkey, int timeout) {
  boolean notimeout = false;
  if (timeout == 0) {
    notimeout = true;
    timeout = 2;
  }
  for (int i = 0; i < timeout; i++) {
    if (digitRead(keytowait)) {
      while (true) {
        if (!digitRead(keytowait)) {
          return 1;
        }
      }
    }
    if (digitRead(abortkey)) {
      while (true) {
        if (!digitRead(abortkey)) {
          return 2;
        }
      }
    }
    if (!notimeout) {
      delay(1);
    }
    else {
      i = 0;
    }
  }
  return 0;
}

//Digitales Lesen für Analog- & Digitalports
boolean digitRead(int key) {
  int r = analogRead(key);
  boolean r2 = digitalRead(key);
  if ((r > 500) | r2) {
    return true;
  }
  return false;
}

//PROGMEM-Symbole in Ram schreiben
void convertChar(const byte* P_Char) {
  for (int i = 0; i < 8; i++) {
    Char[i] = pgm_read_byte(&(P_Char[i]));
  }
}

//EEPROM löschen
void EEPROMClearAll() {
  for (int i = 0 ; i < EEPROM.length() ; i++) {
    EEPROM.write(i, byte(0xFF));
  }  
}

//EEPROM schreiben
void EEPROMWriteInt(int p_address, int p_value)
{
  //p_address = p_address * 2;
  /*
  byte lowByte = ((p_value >> 0) & 0xFF);
  byte highByte = ((p_value >> 8) & 0xFF);

  EEPROM.write(p_address, lowByte);
  EEPROM.write(p_address + 1, highByte);
  */
  EEPROM.write(p_address, p_value);
}


//EEPROM lesen
unsigned int EEPROMReadInt(int p_address)
{
  //p_address = p_address * 2;
  /*
  byte lowByte = EEPROM.read(p_address);
  byte highByte = EEPROM.read(p_address + 1);

  return ((lowByte << 0) & 0xFF) + ((highByte << 8) & 0xFF00);
  */
  byte value = EEPROM.read(p_address);
  if(value == byte(0xFF)){
    return 0;  
  }
  return int(value);
}
